/* ************************************************************************** 
 *             The Pebbles Personal Digital Assistant Project               *
 * **************************************************************************
 * This code was written as part of the Pebbles project at                  *
 * Carnegie Mellon University.  See the following for more information:     *
 *                http://www.cs.cmu.edu/~pebbles                            *
 * This code is being made available for free general non-commercial use.   *
 * For other conditions, see http://www.cs.cmu.edu/~pebbles/conditions.html *
 * Please contact us for more information: pebbles@cs.cmu.edu               *  
 *         Copyright (C) 1998 - Carnegie Mellon University                  *
 * **************************************************************************/

/* 
   This file contains the Pilot version of Ping.
   Ping is a sample Pebbles program that triggers a beep from the PC
   when the user presses a button on the handheld app.
   
   Designed and implemented by Rob Miller.
*/

#include <PalmOS.h>
#include <Pebbles.h>
#include <PebblesPilot.h>
#include "..\PingMessages.h"
#include "PingRsc.h"
#include "PebblesResource.h"

static UInt16    StartApplication(void);
static Err     RomVersionCompatible(UInt32,UInt16);
static void    StopApplication(void);
static Boolean HandleMenuCommand(UInt16);
static Boolean MainFormHandleEvent(EventPtr);
static Boolean ApplicationHandleEvent(EventPtr);
static void    EventLoop(void);
UInt32	       PilotMain(UInt16,void *,UInt16);

PebblesConnection	pbl;


// Application's creator ID and version number.  Note: if you change these, you 
// should make corresponding changes in the Metrowerks project settings (under PilotRez).
#define APPID		'PING'
#define APPVERSION	2


// name of Pebbles plugin DLL corresponding to this app
#define PLUGIN_NAME		"Ping"

// interbyte timeout for receiving messages, in timer ticks (1/100 sec)
#define TIMEOUT			10

/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 * DESCRIPTION:  Initializes serial port connection to Pebbles
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     err - error code for serial initialization
 *
 *
 ***********************************************************************/
static UInt16 StartApplication(void) {
  // open the connection to the Pebbles desktop, asking for our plugin
  if (!PebblesOpen (&pbl, PLUGIN_NAME))
  	return pbl.lastErr;

  return 0;
}

/***********************************************************************
 *
 * FUNCTION:    StopApplication
 *
 * DESCRIPTION:  Shuts off the connection with the PC and closes
 *						the Serial Library
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 *
 ***********************************************************************/
static void StopApplication (void) {
	PebblesClose (&pbl);
}

/***********************************************************************
 *
 * FUNCTION:    ReceiveCheck
 *
 * DESCRIPTION: Polls for messages from the PC.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 *
 ***********************************************************************/
static void ReceiveCheck (void) {
	unsigned len;
	
	switch (PebblesReceiveHeader (&pbl, &len, 0)) {
	case CMD_PONG:
		// received a Pong message -- beep in response
		SndPlaySystemSound (sndInfo);
		
		// Read the message's data.
		// We don't care about the data, so pass a NULL pointer to throw
		// away the received bytes.
		PebblesReceiveData (&pbl, NULL, len, TIMEOUT);
		break;
	}
}


/***********************************************************************
 *
 * FUNCTION:    HandleMenuCommand
 *
 * DESCRIPTION:  Handle the menu commands
 *
 * PARAMETERS:  cmd - menu operation to be performed
 *
 * RETURNED:   boolean value indicating if the event was handled
 *
 *
 ***********************************************************************/
static Boolean HandleMenuCommand(UInt16 cmd) {

  Boolean handled = false;
  FormPtr frm;
  EventType newEvent;

  switch (cmd) {
    case OptionsAboutPing:
      FrmAlert (AboutAlert);
      handled = true;
      break;
    case OptionsConnection:
      FrmPopupForm(ConnectionForm);
      handled = true;
      break;
  }
  return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION:  Handle events in the main commander form
 *
 * PARAMETERS:  event - event to handle
 *
 * RETURNED:    boolean value indicating if the event was handled
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr event) {

  Boolean handled = false;
  FormPtr frm;
  EventType newEvent;
  int id;
	
  switch (event->eType) {
    case frmOpenEvent:
      frm = FrmGetActiveForm ();
      FrmDrawForm (frm);
      handled = true;
      break;
    case ctlSelectEvent:
      switch (event->data.ctlSelect.controlID) {
    	case MainPingButton:
    		// send a PING message to PC
    		PebblesSend (&pbl, CMD_PING, "hello", 5);
    		break;
      }
      break;
    case menuEvent:		
      handled = HandleMenuCommand(event->data.menu.itemID);
  }
  return handled;
}

Boolean ConnFormHandleEvent(EventPtr event) {
	return PebblesConnectionFormHandleEvent(&pbl, event);
}

/***********************************************************************
 *
 * FUNCTION:    ApplicationHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and sets the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *
 ***********************************************************************/	
static Boolean ApplicationHandleEvent(EventPtr event) {

  UInt16 formID;
  FormPtr frm;
  Boolean handled = false;

  switch (event->eType) {
    case frmLoadEvent:
      formID = event->data.frmLoad.formID;
      frm = FrmInitForm (formID);
      FrmSetActiveForm (frm);		
      switch (formID) {
        case MainForm:
          FrmSetEventHandler(frm, MainFormHandleEvent);
          handled = true;
          break;
        case ConnectionForm:
          FrmSetEventHandler(frm, ConnFormHandleEvent);
          handled = true;
          break;
        default:
          break;
      }
      break;
  }
  return handled;
}


/***********************************************************************
 *
 * FUNCTION:    EventLoop
 *
 * DESCRIPTION: This routine is the event loop for the Memo
 *              aplication.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 *
 ***********************************************************************/
static void EventLoop(void) {

  UInt16 error;
  EventType event;
  
  do {
	EvtGetEvent (&event, 100);		// check for serial port activity every 100 ticks

	// If your Pebbles app doesn't need to receive messages from its plugin,
	// then you could simply call PebblesHandshake() here instead of calling 
	// ReceiveCheck ()
	//
  	// PebblesHandshake (&pbl, 0);
  	
  	ReceiveCheck ();
  	
    if (!SysHandleEvent(&event))
      if (!MenuHandleEvent(NULL,&event,&error))
        if (!ApplicationHandleEvent(&event))
          FrmDispatchEvent(&event);
  } while (event.eType != appStopEvent);
}

/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 *
 ***********************************************************************/
UInt32 PilotMain(UInt16 cmd, void * cmdPBP, UInt16 launchFlags) {

  UInt16 error;

  if (cmd == sysAppLaunchCmdNormalLaunch) {
	  error = StartApplication();
	  if (error) return (error);
      FrmGotoForm(MainForm);
	  EventLoop();
	  StopApplication();
  }
  else if (cmd == sysAppLaunchCmdSaveData) {
    FrmSaveAllForms();
  }
  return (0);
}

